@@ -2,6 +2,7 @@
 
 require 'puppet'
 require 'puppet/util/autoload'
+require 'puppet/file_collection/lookup'
 
 # The base class for all of the objects that make up the parse trees.
 # Handles things like file name, line #, and also does the initialization
@@ -10,9 +11,26 @@ class Puppet::Parser::AST
     # Do this so I don't have to type the full path in all of the subclasses
     AST = Puppet::Parser::AST
 
+    include Puppet::FileCollection::Lookup
+
     include Puppet::Util::Errors
     include Puppet::Util::MethodHelper
-    attr_accessor :line, :file, :parent, :scope
+    include Puppet::Util::Docs
+
+    attr_accessor :parent, :scope
+
+    # don't fetch lexer comment by default
+    def use_docs
+        self.class.use_docs
+    end
+
+    # allow our subclass to specify they want documentation
+    class << self
+       attr_accessor :use_docs
+       def associates_doc
+         self.use_docs = true
+       end
+    end
 
     # Does this ast object set something?  If so, it gets evaluated first.
     def self.settor?
@@ -67,26 +85,31 @@ class Puppet::Parser::AST
     # method for them.  This is probably pretty inefficient and should
     # likely be changed at some point.
     def initialize(args)
-        @file = nil
-        @line = nil
         set_options(args)
     end
 end
 
 # And include all of the AST subclasses.
+require 'puppet/parser/ast/arithmetic_operator'
 require 'puppet/parser/ast/astarray'
 require 'puppet/parser/ast/branch'
+require 'puppet/parser/ast/boolean_operator'
 require 'puppet/parser/ast/caseopt'
 require 'puppet/parser/ast/casestatement'
 require 'puppet/parser/ast/collection'
 require 'puppet/parser/ast/collexpr'
+require 'puppet/parser/ast/comparison_operator'
 require 'puppet/parser/ast/definition'
 require 'puppet/parser/ast/else'
 require 'puppet/parser/ast/function'
 require 'puppet/parser/ast/hostclass'
 require 'puppet/parser/ast/ifstatement'
 require 'puppet/parser/ast/leaf'
+require 'puppet/parser/ast/match_operator'
+require 'puppet/parser/ast/minus'
 require 'puppet/parser/ast/node'
+require 'puppet/parser/ast/nop'
+require 'puppet/parser/ast/not'
 require 'puppet/parser/ast/resource'
 require 'puppet/parser/ast/resource_defaults'
 require 'puppet/parser/ast/resource_override'
