@@ -3,6 +3,9 @@ require 'puppet/parser/ast/branch'
 class Puppet::Parser::AST
     # A basic 'if/elsif/else' statement.
     class IfStatement < AST::Branch
+
+        associates_doc
+
         attr_accessor :test, :else, :statements
 
         def each
@@ -13,16 +16,22 @@ class Puppet::Parser::AST
         # else if there's an 'else' setting, evaluate it.
         # the first option that matches.
         def evaluate(scope)
+            level = scope.ephemeral_level
             value = @test.safeevaluate(scope)
 
-            if Puppet::Parser::Scope.true?(value)
-                return @statements.safeevaluate(scope)
-            else
-                if defined? @else
-                    return @else.safeevaluate(scope)
+            # let's emulate a new scope for each branches
+            begin
+                if Puppet::Parser::Scope.true?(value)
+                    return @statements.safeevaluate(scope)
                 else
-                    return nil
+                    if defined? @else
+                        return @else.safeevaluate(scope)
+                    else
+                        return nil
+                    end
                 end
+            ensure
+                scope.unset_ephemeral_var(level)
             end
         end
     end
