@@ -15,9 +15,34 @@ class Puppet::Parser::Scope
 
     include Enumerable
     include Puppet::Util::Errors
-    attr_accessor :parent, :level, :parser, :source, :resource
+    attr_accessor :level, :parser, :source, :resource
     attr_accessor :base, :keyword, :nodescope
     attr_accessor :top, :translated, :compiler
+    attr_writer :parent
+
+    # thin wrapper around an ephemeral
+    # symbol table.
+    # when a symbol
+    class Ephemeral
+        def initialize(parent=nil)
+            @symbols = {}
+            @parent = parent
+        end
+
+        [:include?, :delete, :[]=].each do |m|
+            define_method(m) do |*args|
+                @symbols.send(m, *args)
+            end
+        end
+
+        def [](name)
+            unless @symbols.include?(name) or @parent.nil?
+                @parent[name]
+            else
+                @symbols[name]
+            end
+        end
+    end
 
     # A demeterific shortcut to the catalog.
     def catalog
@@ -43,6 +68,29 @@ class Puppet::Parser::Scope
         end
     end
 
+    # Is the value a number?, return the correct object or nil if not a number
+    def self.number?(value)
+        unless value.is_a?(Fixnum) or value.is_a?(Bignum) or value.is_a?(Float) or value.is_a?(String)
+            return nil
+        end
+
+        if value.is_a?(String)
+            if value =~ /^-?\d+(:?\.\d+|(:?\.\d+)?e\d+)$/
+                return value.to_f
+            elsif value =~ /^0x[0-9a-f]+$/i
+                return value.to_i(16)
+            elsif value =~ /^0[0-7]+$/
+                return value.to_i(8)
+            elsif value =~ /^-?\d+$/
+                return value.to_i
+            else
+                return nil
+            end
+        end
+        # it is one of Fixnum,Bignum or Float
+        return value
+    end
+
     # Add to our list of namespaces.
     def add_namespace(ns)
         return false if @namespaces.include?(ns)
@@ -58,18 +106,18 @@ class Puppet::Parser::Scope
         @level == 1
     end
 
-    def findclass(name)
+    def find_hostclass(name)
         @namespaces.each do |namespace|
-            if r = parser.findclass(namespace, name)
+            if r = parser.find_hostclass(namespace, name)
                 return r
             end
         end
         return nil
     end
 
-    def finddefine(name)
+    def find_definition(name)
         @namespaces.each do |namespace|
-            if r = parser.finddefine(namespace, name)
+            if r = parser.find_definition(namespace, name)
                 return r
             end
         end
@@ -104,6 +152,13 @@ class Puppet::Parser::Scope
         # The symbol table for this scope.  This is where we store variables.
         @symtable = {}
 
+        # the ephemeral symbol tables
+        # those should not persist long, and are used for the moment only
+        # for $0..$xy capture variables of regexes
+        # this is actually implemented as a stack, with each ephemeral scope
+        # shadowing the previous one
+        @ephemeral = [ Ephemeral.new ]
+
         # All of the defaults set for types.  It's a hash of hashes,
         # with the first key being the type, then the second key being
         # the parameter.
@@ -141,19 +196,21 @@ class Puppet::Parser::Scope
 
     # Look up a defined type.
     def lookuptype(name)
-        finddefine(name) || findclass(name)
+        find_definition(name) || find_hostclass(name)
     end
 
     def lookup_qualified_var(name, usestring)
         parts = name.split(/::/)
         shortname = parts.pop
         klassname = parts.join("::")
-        klass = findclass(klassname)
+        klass = find_hostclass(klassname)
         unless klass
-            raise Puppet::ParseError, "Could not find class %s" % klassname
+            warning "Could not look up qualified variable '%s'; class %s could not be found" % [name, klassname]
+            return usestring ? "" : :undefined
         end
         unless kscope = compiler.class_scope(klass)
-            raise Puppet::ParseError, "Class %s has not been evaluated so its variables cannot be referenced" % klass.classname
+            warning "Could not look up qualified variable '%s'; class %s has not been evaluated" % [name, klassname]
+            return usestring ? "" : :undefined
         end
         return kscope.lookupvar(shortname, usestring)
     end
@@ -163,18 +220,19 @@ class Puppet::Parser::Scope
     # Look up a variable.  The simplest value search we do.  Default to returning
     # an empty string for missing values, but support returning a constant.
     def lookupvar(name, usestring = true)
+        table = ephemeral?(name) ? @ephemeral.last : @symtable
         # If the variable is qualified, then find the specified scope and look the variable up there instead.
         if name =~ /::/
             return lookup_qualified_var(name, usestring)
         end
-        # We can't use "if @symtable[name]" here because the value might be false
-        if @symtable.include?(name)
-            if usestring and @symtable[name] == :undef
+        # We can't use "if table[name]" here because the value might be false
+        if ephemeral_include?(name) or table.include?(name)
+            if usestring and table[name] == :undef
                 return ""
             else
-                return @symtable[name]
+                return table[name]
             end
-        elsif self.parent 
+        elsif self.parent
             return parent.lookupvar(name, usestring)
         elsif usestring
             return ""
@@ -183,6 +241,25 @@ class Puppet::Parser::Scope
         end
     end
 
+    # Return a hash containing our variables and their values, optionally (and
+    # by default) including the values defined in our parent.  Local values
+    # shadow parent values.
+    def to_hash(recursive = true)
+        if recursive and parent then
+            target = parent.to_hash(recursive)
+        end
+        target ||= Hash.new
+        @symtable.keys.each { |name|
+            value = @symtable[name]
+            if value == :undef then
+                target.delete(name)
+            else
+                target[name] = value
+            end
+        }
+        return target
+    end
+
     def namespaces
         @namespaces.dup
     end
@@ -241,20 +318,37 @@ class Puppet::Parser::Scope
     # Set a variable in the current scope.  This will override settings
     # in scopes above, but will not allow variables in the current scope
     # to be reassigned.
-    def setvar(name,value, file = nil, line = nil)
-        #Puppet.debug "Setting %s to '%s' at level %s" %
-        #    [name.inspect,value,self.level]
-        if @symtable.include?(name)
-            error = Puppet::ParseError.new("Cannot reassign variable %s" % name)
-            if file
-                error.file = file
+    def setvar(name,value, options = {})
+        table = options[:ephemeral] ? @ephemeral.last : @symtable
+        #Puppet.debug "Setting %s to '%s' at level %s mode append %s" %
+        #    [name.inspect,value,self.level, append]
+        if table.include?(name)
+            unless options[:append]
+                error = Puppet::ParseError.new("Cannot reassign variable %s" % name)
+            else
+                error = Puppet::ParseError.new("Cannot append, variable %s is defined in this scope" % name)
             end
-            if line
-                error.line = line
+            if options[:file]
+                error.file = options[:file]
+            end
+            if options[:line]
+                error.line = options[:line]
             end
             raise error
         end
-        @symtable[name] = value
+
+        unless options[:append]
+            table[name] = value
+        else # append case
+            # lookup the value in the scope if it exists and insert the var
+            table[name] = lookupvar(name)
+            # concatenate if string, append if array, nothing for other types
+            if value.is_a?(Array)
+                table[name] += value
+            else
+                table[name] << value
+            end
+        end
     end
 
     # Return an interpolated string.
@@ -263,12 +357,17 @@ class Puppet::Parser::Scope
         ss = StringScanner.new(string)
         out = ""
         while not ss.eos?
-            if ss.scan(/^\$\{((\w*::)*\w+)\}|^\$((\w*::)*\w+)/) 
+            if ss.scan(/^\$\{((\w*::)*\w+|[0-9]+)\}|^\$([0-9])|^\$((\w*::)*\w+)/)
                 # If it matches the backslash, then just retun the dollar sign.
                 if ss.matched == '\\$'
                     out << '$'
                 else # look the variable up
-                    out << lookupvar(ss[1] || ss[3]).to_s || ""
+                    # make sure $0-$9 are lookupable only if ephemeral
+                    var = ss[1] || ss[3] || ss[4]
+                    if var and var =~ /^[0-9]+$/ and not ephemeral_include?(var)
+                        next
+                    end
+                    out << lookupvar(var).to_s || ""
                 end
             elsif ss.scan(/^\\(.)/)
                 # Puppet.debug("Got escape: pos:%d; m:%s" % [ss.pos, ss.matched])
@@ -298,7 +397,7 @@ class Puppet::Parser::Scope
                 out << '$'
             elsif ss.scan(/^\\\n/) # an escaped carriage return
                 next
-            else 
+            else
                 tmp = ss.scan(/[^\\$]+/)
                 # Puppet.debug("Got other: pos:%d; m:%s" % [ss.pos, tmp])
                 unless tmp
@@ -330,8 +429,52 @@ class Puppet::Parser::Scope
 
     # Undefine a variable; only used for testing.
     def unsetvar(var)
-        if @symtable.include?(var)
-            @symtable.delete(var)
+        table = ephemeral?(var) ? @ephemeral.last : @symtable
+        if table.include?(var)
+            table.delete(var)
+        end
+    end
+
+    # remove ephemeral scope up to level
+    def unset_ephemeral_var(level=:all)
+        if level == :all
+            @ephemeral = [ Ephemeral.new ]
+        else
+            (@ephemeral.size - level).times do 
+                @ephemeral.pop
+            end
+        end
+    end
+
+    # check if name exists in one of the ephemeral scope.
+    def ephemeral_include?(name)
+        @ephemeral.reverse.each do |eph|
+            return true if eph.include?(name)
+        end
+        false
+    end
+
+    # is name an ephemeral variable?
+    def ephemeral?(name)
+        name =~ /^\d+$/
+    end
+
+    def ephemeral_level
+        @ephemeral.size
+    end
+
+    def new_ephemeral
+        @ephemeral.push(Ephemeral.new(@ephemeral.last))
+    end
+
+    def ephemeral_from(match, file = nil, line = nil)
+        raise(ArgumentError,"Invalid regex match data") unless match.is_a?(MatchData)
+
+        new_ephemeral
+
+        setvar("0", match[0], :file => file, :line => line, :ephemeral => true)
+        match.captures.each_with_index do |m,i|
+            setvar("#{i+1}", m, :file => file, :line => line, :ephemeral => true)
         end
     end
 end
