@@ -3,7 +3,7 @@ require 'puppet/provider/mount'
 
 fstab = nil
 case Facter.value(:operatingsystem)
-when "Solaris": fstab = "/etc/vfstab"
+when "Solaris"; fstab = "/etc/vfstab"
 else
     fstab = "/etc/fstab"
 end
@@ -14,13 +14,13 @@ Puppet::Type.type(:mount).provide(:parsed,
     :filetype => :flat
 ) do
     include Puppet::Provider::Mount
-    confine :exists => fstab
+    #confine :exists => fstab
 
-    commands :mountcmd => "mount", :umount => "umount", :df => "df"
+    commands :mountcmd => "mount", :umount => "umount"
 
     @platform = Facter["operatingsystem"].value
     case @platform
-    when "Solaris":
+    when "Solaris"
         @fields = [:device, :blockdevice, :name, :fstype, :pass, :atboot,
                :options]
     else
@@ -31,7 +31,15 @@ Puppet::Type.type(:mount).provide(:parsed,
     text_line :comment, :match => /^\s*#/
     text_line :blank, :match => /^\s*$/
 
-    record_line self.name, :fields => @fields, :separator => /\s+/, :joiner => "\t"
+    optional_fields  = @fields - [:device, :name, :blockdevice]
+    mandatory_fields = @fields - optional_fields
+
+    # fstab will ignore lines that have fewer than the mandatory number of columns,
+    # so we should, too.
+    field_pattern = '(\s*(?>\S+))'
+    text_line :incomplete, :match => /^(?!#{field_pattern}{#{mandatory_fields.length}})/
+
+    record_line self.name, :fields => @fields, :separator => /\s+/, :joiner => "\t", :optional => optional_fields
+
 end
 
-# $Id: parsed.rb 1860 2006-11-12 21:25:13Z luke $
