@@ -33,7 +33,7 @@ Puppet::Type.type(:package).provide :ports, :parent => :freebsd, :source => :fre
         begin
             output = portversion(*cmd)
         rescue Puppet::ExecutionFailure
-            raise Puppet::PackageError.new(output)
+            raise Puppet::Error.new(output)
         end
         line = output.split("\n").pop
 
@@ -47,13 +47,13 @@ Puppet::Type.type(:package).provide :ports, :parent => :freebsd, :source => :fre
         info = $3
 
         unless pkgstuff =~ /^(\S+)-([^-\s]+)$/
-            raise Puppet::PackageError,
+            raise Puppet::Error,
                 "Could not match package info '%s'" % pkgstuff
         end
 
         name, version = $1, $2
 
-        if match == "=" or match == ">" 
+        if match == "=" or match == ">"
             # we're up to date or more recent
             return version
         end
@@ -61,7 +61,7 @@ Puppet::Type.type(:package).provide :ports, :parent => :freebsd, :source => :fre
         # Else, we need to be updated; we need to pull out the new version
 
         unless info =~ /\((\w+) has (.+)\)/
-            raise Puppet::PackageError,
+            raise Puppet::Error,
                 "Could not match version info '%s'" % info
         end
 
@@ -72,8 +72,13 @@ Puppet::Type.type(:package).provide :ports, :parent => :freebsd, :source => :fre
     end
 
     def query
+        # support portorigin_glob such as "mail/postfix"
+        name = self.name
+        if name =~ /\//
+            name = self.name.split(/\//).slice(1)
+        end
         self.class.instances.each do |instance|
-            if instance.name == self.name
+            if instance.name == name
                 return instance.properties
             end
         end
