@@ -1,7 +1,12 @@
 # Manage FreeBSD services.
 Puppet::Type.type(:service).provide :freebsd, :parent => :init do
-    desc "FreeBSD's (and probably NetBSD?) form of ``init``-style service
-        management; uses ``rc.conf.d`` for service enabling and disabling."
+    desc "FreeBSD's (and probably NetBSD?) form of ``init``-style service management.
+
+    Uses ``rc.conf.d`` for service enabling and disabling.
+
+"
+
+    confine :operatingsystem => [:freebsd, :netbsd, :openbsd]
 
     defaultfor :operatingsystem => :freebsd
 
@@ -10,7 +15,7 @@ Puppet::Type.type(:service).provide :freebsd, :parent => :init do
     def self.defpath
         superclass.defpath
     end
-    
+
     # remove service file from rc.conf.d to disable it
     def disable
         rcfile = File.join(@@rcconf_dir, @model[:name])
@@ -18,7 +23,7 @@ Puppet::Type.type(:service).provide :freebsd, :parent => :init do
             File.delete(rcfile)
         end
     end
-    
+
     # if the service file exists in rc.conf.d then it's already enabled
     def enabled?
         rcfile = File.join(@@rcconf_dir, @model[:name])
@@ -28,7 +33,7 @@ Puppet::Type.type(:service).provide :freebsd, :parent => :init do
 
         return :false
     end
-    
+
     # enable service by creating a service file under rc.conf.d with the
     # proper contents
     def enable
@@ -38,7 +43,7 @@ Puppet::Type.type(:service).provide :freebsd, :parent => :init do
         rcfile = File.join(@@rcconf_dir, @model[:name])
         open(rcfile, 'w') { |f| f << "%s_enable=\"YES\"\n" % @model[:name] }
     end
-    
+
     # Override stop/start commands to use one<cmd>'s and the avoid race condition
     # where provider trys to stop/start the service before it is enabled
     def startcmd
