@@ -4,6 +4,7 @@ require 'facter'
 require 'puppet'
 
 module Puppet::Rails
+    TIME_DEBUG = true
 
     def self.connect
         # This global init does not work for testing, because we remove
@@ -21,34 +22,40 @@ module Puppet::Rails
             ActiveRecord::Base.logger.level = Logger::DEBUG
         end
 
-        ActiveRecord::Base.allow_concurrency = true
+        if (::ActiveRecord::VERSION::MAJOR == 2 and ::ActiveRecord::VERSION::MINOR <= 1)
+            ActiveRecord::Base.allow_concurrency = true
+        end
+
         ActiveRecord::Base.verify_active_connections!
 
         begin
-            ActiveRecord::Base.establish_connection(database_arguments())
+            args = database_arguments
+            Puppet.info "Connecting to #{args[:adapter]} database: #{args[:database]}"
+            ActiveRecord::Base.establish_connection(args)
         rescue => detail
             if Puppet[:trace]
                 puts detail.backtrace
             end
             raise Puppet::Error, "Could not connect to database: %s" % detail
-        end 
+        end
     end
 
     # The arguments for initializing the database connection.
     def self.database_arguments
         adapter = Puppet[:dbadapter]
-        
+
         args = {:adapter => adapter, :log_level => Puppet[:rails_loglevel]}
 
         case adapter
-        when "sqlite3":
-            args[:dbfile] = Puppet[:dblocation]
-        when "mysql", "postgresql":
+        when "sqlite3"
+            args[:database] = Puppet[:dblocation]
+        when "mysql", "postgresql"
             args[:host]     = Puppet[:dbserver] unless Puppet[:dbserver].empty?
             args[:username] = Puppet[:dbuser] unless Puppet[:dbuser].empty?
             args[:password] = Puppet[:dbpassword] unless Puppet[:dbpassword].empty?
             args[:database] = Puppet[:dbname]
-            
+            args[:reconnect]= true
+
             socket          = Puppet[:dbsocket]
             args[:socket]   = socket unless socket.empty?
         else
@@ -93,7 +100,7 @@ module Puppet::Rails
 
         unless ActiveRecord::Base.connection.tables.include?("resources")
             raise Puppet::Error, "Database has problems, can't migrate."
-        end    
+        end
 
         Puppet.notice "Migrating"
 
@@ -122,9 +129,9 @@ module Puppet::Rails
                puts detail.backtrace
             end
             raise Puppet::Error, "Could not connect to database: %s" % detail
-        end 
+        end
 
-        ActiveRecord::Base.connection.tables.each do |t| 
+        ActiveRecord::Base.connection.tables.each do |t|
             ActiveRecord::Base.connection.drop_table t
         end
     end
