@@ -2,10 +2,13 @@
 # and performs them
 
 require 'puppet'
-require 'puppet/propertychange'
+require 'puppet/util/tagging'
 
 module Puppet
 class Transaction
+    require 'puppet/transaction/change'
+    require 'puppet/transaction/event'
+
     attr_accessor :component, :catalog, :ignoreschedules
     attr_accessor :sorted_resources, :configurator
 
@@ -14,8 +17,9 @@ class Transaction
 
     # The list of events generated in this transaction.
     attr_reader :events
-    
+
     include Puppet::Util
+    include Puppet::Util::Tagging
 
     # Add some additional times for reporting
     def addtimes(hash)
@@ -33,7 +37,7 @@ class Transaction
         if resource.purging? and resource.deleting?
             if deps = relationship_graph.dependents(resource) and ! deps.empty? and deps.detect { |d| ! d.deleting? }
                 resource.warning "%s still depend%s on me -- not purging" %
-                    [deps.collect { |r| r.ref }.join(","), deps.length > 1 ? "":"s"] 
+                    [deps.collect { |r| r.ref }.join(","), deps.length > 1 ? "":"s"]
                 return false
             end
         end
@@ -89,14 +93,14 @@ class Transaction
             if resource.respond_to?(:flush)
                 resource.flush
             end
-            
+
             # And set a trigger for refreshing this resource if it's a
             # self-refresher
             if resource.self_refresh? and ! resource.deleting?
                 # Create an edge with this resource as both the source and
                 # target.  The triggering method treats these specially for
                 # logging.
-                events = resourceevents.collect { |e| e.event }
+                events = resourceevents.collect { |e| e.name }
                 set_trigger(Puppet::Relationship.new(resource, resource, :callback => :refresh, :event => events))
             end
         end
@@ -109,7 +113,6 @@ class Transaction
         changes.collect { |change|
             @changes << change
             @count += 1
-            change.transaction = self
             events = nil
             begin
                 # use an array, so that changes can return more than one
@@ -148,27 +151,19 @@ class Transaction
             change.property.resource
         }.uniq
     end
-    
-    # Do any necessary cleanup.  If we don't get rid of the graphs, the
-    # contained resources might never get cleaned up.
-    def cleanup
-        if defined? @generated
-            relationship_graph.remove_resource(*@generated)
-        end
-    end
 
     # Copy an important relationships from the parent to the newly-generated
     # child resource.
-    def copy_relationships(resource, children)
+    def make_parent_child_relationship(resource, children)
         depthfirst = resource.depthfirst?
-        
+
         children.each do |gen_child|
             if depthfirst
                 edge = [gen_child, resource]
             else
                 edge = [resource, gen_child]
             end
-            relationship_graph.add_resource(gen_child) unless relationship_graph.resource(gen_child.ref)
+            relationship_graph.add_vertex(gen_child)
 
             unless relationship_graph.edge?(edge[1], edge[0])
                 relationship_graph.add_edge(*edge)
@@ -187,81 +182,21 @@ class Transaction
 
     # See if the resource generates new resources at evaluation time.
     def eval_generate(resource)
-        if resource.respond_to?(:eval_generate)
-            begin
-                children = resource.eval_generate
-            rescue => detail
-                if Puppet[:trace]
-                    puts detail.backtrace
-                end
-                resource.err "Failed to generate additional resources during transaction: %s" %
-                    detail
-                return nil
-            end
-            
-            if children
-                children.each { |child| child.finish }
-                @generated += children
-                return children
-            end
-        end
+        generate_additional_resources(resource, :eval_generate)
     end
-    
+
     # Evaluate a single resource.
-    def eval_resource(resource, checkskip = true)
+    def eval_resource(resource)
         events = []
-        
+
         if resource.is_a?(Puppet::Type::Component)
             raise Puppet::DevError, "Got a component to evaluate"
         end
-        
-        if checkskip and skip?(resource)
+
+        if skip?(resource)
             @resourcemetrics[:skipped] += 1
         else
-            @resourcemetrics[:scheduled] += 1
-            
-            changecount = @changes.length
-            
-            # We need to generate first regardless, because the recursive
-            # actions sometimes change how the top resource is applied.
-            children = eval_generate(resource)
-            
-            if children and resource.depthfirst?
-                children.each do |child|
-                    # The child will never be skipped when the parent isn't
-                    events += eval_resource(child, false)
-                end
-            end
-
-            # Perform the actual changes
-            seconds = thinmark do
-                events += apply(resource)
-            end
-
-            if children and ! resource.depthfirst?
-                children.each do |child|
-                    events += eval_resource(child, false)
-                end
-            end
-
-            # Create a child/parent relationship.  We do this after everything else because
-            # we want explicit relationships to be able to override automatic relationships,
-            # including this one.
-            if children
-                copy_relationships(resource, children)
-            end
-            
-            # A bit of hackery here -- if skipcheck is true, then we're the
-            # top-level resource.  If that's the case, then make sure all of
-            # the changes list this resource as a proxy.  This is really only
-            # necessary for rollback, since we know the generating resource
-            # during forward changes.
-            if children and checkskip
-                @changes[changecount..-1].each { |change| change.proxy = resource }
-            end
-
-            # Keep track of how long we spend in each type of resource
-            @timemetrics[resource.class.name] += seconds
+            events += eval_children_and_apply_resource(resource)
         end
 
         # Check to see if there are any events for this resource
@@ -272,21 +207,67 @@ class Transaction
         # Collect the targets of any subscriptions to those events.  We pass
         # the parent resource in so it will override the source in the events,
         # since eval_generated children can't have direct relationships.
-        relationship_graph.matching_edges(events, resource).each do |orig_edge|
-            # We have to dup the label here, else we modify the original edge label,
-            # which affects whether a given event will match on the next run, which is,
-            # of course, bad.
-            edge = orig_edge.class.new(orig_edge.source, orig_edge.target)
-            label = orig_edge.label.dup
-            label[:event] = events.collect { |e| e.event }
-            edge.label = label
-            set_trigger(edge)
+        duration = thinmark do
+            b = relationship_graph.matching_edges(events, resource)
+            b.each do |orig_edge|
+                # We have to dup the label here, else we modify the original edge label,
+                # which affects whether a given event will match on the next run, which is,
+                # of course, bad.
+                edge = orig_edge.class.new(orig_edge.source, orig_edge.target, orig_edge.label)
+                edge.event = events.collect { |e| e.name }
+                set_trigger(edge)
+            end
         end
+        Puppet.debug("Time for triggering #{events.size} events to edges: #{duration}") if events.size > 0 and duration > 0
 
         # And return the events for collection
         events
     end
 
+    def eval_children_and_apply_resource(resource)
+        events = []
+
+        @resourcemetrics[:scheduled] += 1
+
+        changecount = @changes.length
+
+        # We need to generate first regardless, because the recursive
+        # actions sometimes change how the top resource is applied.
+        children = eval_generate(resource)
+
+        if ! children.empty? and resource.depthfirst?
+            children.each do |child|
+                # The child will never be skipped when the parent isn't
+                events += eval_resource(child, false)
+            end
+        end
+
+        # Perform the actual changes
+        seconds = thinmark do
+            events += apply(resource)
+        end
+
+        if ! children.empty? and ! resource.depthfirst?
+            children.each do |child|
+                events += eval_resource(child)
+            end
+        end
+
+        # A bit of hackery here -- if skipcheck is true, then we're the
+        # top-level resource.  If that's the case, then make sure all of
+        # the changes list this resource as a proxy.  This is really only
+        # necessary for rollback, since we know the generating resource
+        # during forward changes.
+        unless children.empty?
+            @changes[changecount..-1].each { |change| change.proxy = resource }
+        end
+
+        # Keep track of how long we spend in each type of resource
+        @timemetrics[resource.class.name] += seconds
+
+        events
+    end
+
     # This method does all the actual work of running a transaction.  It
     # collects all of the changes, executes them, and responds to any
     # necessary events.
@@ -295,9 +276,11 @@ class Transaction
 
         # Start logging.
         Puppet::Util::Log.newdestination(@report)
-        
+
         prepare()
 
+        Puppet.info "Applying configuration version '%s'" % catalog.version if catalog.version
+
         begin
             allevents = @sorted_resources.collect { |resource|
                 if resource.is_a?(Puppet::Type::Component)
@@ -350,43 +333,46 @@ class Transaction
                 skip = true
             end
         end
-        
+
         return skip
     end
-    
-    # Collect any dynamically generated resources.
+
+    # A general method for recursively generating new resources from a
+    # resource.
+    def generate_additional_resources(resource, method)
+        return [] unless resource.respond_to?(method)
+        begin
+            made = resource.send(method)
+        rescue => detail
+            puts detail.backtrace if Puppet[:trace]
+            resource.err "Failed to generate additional resources using '%s': %s" % [method, detail]
+        end
+        return [] unless made
+        made = [made] unless made.is_a?(Array)
+        made.uniq.find_all do |res|
+            begin
+                res.tag(*resource.tags)
+                @catalog.add_resource(res) do |r|
+                    r.finish
+                    make_parent_child_relationship(resource, [r])
+                end
+                true
+            rescue Puppet::Resource::Catalog::DuplicateResourceError
+                res.info "Duplicate generated resource; skipping"
+                false
+            end
+        end
+    end
+
+    # Collect any dynamically generated resources.  This method is called
+    # before the transaction starts.
     def generate
         list = @catalog.vertices
-        
-        # Store a list of all generated resources, so that we can clean them up
-        # after the transaction closes.
-        @generated = []
-        
         newlist = []
         while ! list.empty?
             list.each do |resource|
-                if resource.respond_to?(:generate)
-                    begin
-                        made = resource.generate
-                    rescue => detail
-                        resource.err "Failed to generate additional resources: %s" %
-                            detail
-                    end
-                    next unless made
-                    unless made.is_a?(Array)
-                        made = [made]
-                    end
-                    made.uniq!
-                    made.each do |res|
-                        @catalog.add_resource(res)
-                        res.catalog = catalog
-                        newlist << res
-                        @generated << res
-                        res.finish
-                    end
-                end
+                newlist += generate_additional_resources(resource, :generate)
             end
-            list.clear
             list = newlist
             newlist = []
         end
@@ -428,10 +414,10 @@ class Transaction
     # this should only be called by a Puppet::Type::Component resource now
     # and it should only receive an array
     def initialize(resources)
-        if resources.is_a?(Puppet::Node::Catalog)
+        if resources.is_a?(Puppet::Resource::Catalog)
             @catalog = resources
-        elsif resources.is_a?(Puppet::PGraph)
-            raise "Transactions should get catalogs now, not PGraph"
+        elsif resources.is_a?(Puppet::SimpleGraph)
+            raise "Transactions should get catalogs now, not SimpleGraph"
         else
             raise "Transactions require catalogs"
         end
@@ -479,7 +465,7 @@ class Transaction
         @catalog.vertices.each do |resource|
             if provider = resource.provider and provider.class.respond_to?(:prefetch)
                 prefetchers[provider.class] ||= {}
-                prefetchers[provider.class][resource.title] = resource
+                prefetchers[provider.class][resource.name] = resource
             end
         end
 
@@ -496,14 +482,16 @@ class Transaction
             end
         end
     end
-    
+
     # Prepare to evaluate the resources in a transaction.
     def prepare
-        prefetch()
-    
         # Now add any dynamically generated resources
         generate()
-        
+
+        # Then prefetch.  It's important that we generate and then prefetch,
+        # so that any generated resources also get prefetched.
+        prefetch()
+
         # This will throw an error if there are cycles in the graph.
         @sorted_resources = relationship_graph.topsort
     end
@@ -511,7 +499,7 @@ class Transaction
     def relationship_graph
         catalog.relationship_graph
     end
-    
+
     # Send off the transaction report.
     def send_report
         begin
@@ -521,33 +509,19 @@ class Transaction
             return
         end
 
-        if Puppet[:rrdgraph] == true
-            report.graph()
-        end
-
         if Puppet[:summarize]
             puts report.summary
         end
-        
+
         if Puppet[:report]
             begin
-                reportclient().report(report)
+                report.save()
             rescue => detail
                 Puppet.err "Reporting failed: %s" % detail
             end
         end
     end
 
-    def reportclient
-        unless defined? @reportclient
-            @reportclient = Puppet::Network::Client.report.new(
-                :Server => Puppet[:reportserver]
-            )
-        end
-
-        @reportclient
-    end
-
     # Roll all completed changes back.
     def rollback
         @targets.clear
@@ -573,7 +547,7 @@ class Transaction
                 # this still could get hairy; what if file contents changed,
                 # but a chmod failed?  how would i handle that error? dern
             end
-            
+
             # FIXME This won't work right now.
             relationship_graph.matching_edges(events).each do |edge|
                 @targets[edge.target] << edge
@@ -588,7 +562,7 @@ class Transaction
             events
         }.flatten.reject { |e| e.nil? }
     end
-    
+
     # Is the resource currently scheduled?
     def scheduled?(resource)
         self.ignoreschedules or resource.scheduled?
@@ -605,7 +579,7 @@ class Transaction
         end
         @targets[edge.target] << edge
     end
-    
+
     # Should this resource be skipped?
     def skip?(resource)
         skip = false
@@ -615,37 +589,39 @@ class Transaction
             resource.debug "Not scheduled"
         elsif failed_dependencies?(resource)
             resource.warning "Skipping because of failed dependencies"
+        elsif resource.virtual?
+            resource.debug "Skipping because virtual"
         else
             return false
         end
         return true
     end
-    
+
     # The tags we should be checking.
     def tags
         unless defined? @tags
-            tags = Puppet[:tags]
-            if tags.nil? or tags == ""
-                @tags = []
-            else
-                @tags = tags.split(/\s*,\s*/)
-            end
+            self.tags = Puppet[:tags]
         end
-        
-        @tags
+
+        super
     end
 
-    def tags=(tags)
-        tags = [tags] unless tags.is_a?(Array)
-        @tags = tags
+    def handle_qualified_tags( qualified )
+        # The default behavior of Puppet::Util::Tagging is
+        # to split qualified tags into parts. That would cause
+        # qualified tags to match too broadly here.
+        return
     end
-    
+
     # Is this resource tagged appropriately?
     def missing_tags?(resource)
-        return false if self.ignore_tags? or tags.empty?
-        return true unless resource.tagged?(tags)
+        not appropriately_tagged?(resource)
     end
-    
+
+    def appropriately_tagged?(resource)
+        self.ignore_tags? or tags.empty? or resource.tagged?(*tags)
+    end
+
     # Are there any edges that target this resource?
     def targeted?(resource)
         # The default value is a new array so we have to test the length of it.
@@ -678,11 +654,7 @@ class Transaction
                     [callback, subs.length]
 
                 # And then add an event for it.
-                return [Puppet::Event.new(
-                    :event => :noop,
-                    :transaction => self,
-                    :source => resource
-                )]
+                return [Puppet::Transaction::Event.new(:noop, resource)]
             end
 
             if subs.length == 1 and subs[0].source == resource
@@ -692,7 +664,7 @@ class Transaction
                     [callback, subs.length]
             end
             resource.notice message
-            
+
             # At this point, just log failures, don't try to react
             # to them in any way.
             begin
@@ -710,11 +682,7 @@ class Transaction
             end
 
             # And then add an event for it.
-            trigged << Puppet::Event.new(
-                :event => :triggered,
-                :transaction => self,
-                :source => resource
-            )
+            trigged << Puppet::Transaction::Event.new(:triggered, resource)
 
             triggered(resource, callback)
         end
