@@ -1,37 +1,51 @@
 require 'etc'
 require 'facter'
-require 'puppet/type/property'
 require 'puppet/util/filetype'
-require 'puppet/type/parsedtype'
 
 Puppet::Type.newtype(:cron) do
-    @doc = "Installs and manages cron jobs.  All fields except the command 
+    @doc = "Installs and manages cron jobs.  All fields except the command
         and the user are optional, although specifying no periodic
         fields would result in the command being executed every
         minute.  While the name of the cron job is not part of the actual
         job, it is used by Puppet to store and retrieve it.
-        
+
         If you specify a cron job that matches an existing job in every way
         except name, then the jobs will be considered equivalent and the
         new name will be permanently associated with that job.  Once this
         association is made and synced to disk, you can then manage the job
         normally (e.g., change the schedule of the job).
-        
+
         Example::
-            
+
             cron { logrotate:
                 command => \"/usr/sbin/logrotate\",
                 user => root,
                 hour => 2,
                 minute => 0
             }
+
+        Note that all cron values can be specified as an array of values::
+
+            cron { logrotate:
+                command => \"/usr/sbin/logrotate\",
+                user => root,
+                hour => [2, 4]
+            }
+
+        Or using ranges, or the step syntax ``*/2`` (although there's no guarantee that
+        your ``cron`` daemon supports it)::
+
+            cron { logrotate:
+                command => \"/usr/sbin/logrotate\",
+                user => root,
+                hour => ['2-4'],
+                minute => '*/10'
+            }
         "
     ensurable
 
     # A base class for all of the Cron parameters, since they all have
-    # similar argument checking going on.  We're stealing the base class
-    # from parsedtype, and we should probably subclass Cron from there,
-    # but it was just too annoying to do.
+    # similar argument checking going on.
     class CronParam < Puppet::Property
         class << self
             attr_accessor :boundaries, :default
@@ -39,9 +53,9 @@ Puppet::Type.newtype(:cron) do
 
         # We have to override the parent method, because we consume the entire
         # "should" array
-        def insync?
+        def insync?(is)
             if defined? @should and @should
-                self.is_to_s == self.should_to_s
+                self.is_to_s(is) == self.should_to_s
             else
                 true
             end
@@ -75,7 +89,7 @@ Puppet::Type.newtype(:cron) do
         # or the first three letters of the word.
         def alphacheck(value, ary)
             tmp = value.downcase
-            
+
             # If they specified a shortened version of the name, then see
             # if we can lengthen it (e.g., mon => monday).
             if tmp.length == 3
@@ -93,36 +107,31 @@ Puppet::Type.newtype(:cron) do
             return false
         end
 
-        def should_to_s
-            if @should
-                if self.name == :command or @should[0].is_a? Symbol
-                    @should[0]
+        def should_to_s(newvalue = @should)
+            if newvalue
+                unless newvalue.is_a?(Array)
+                    newvalue = [newvalue]
+                end
+                if self.name == :command or newvalue[0].is_a? Symbol
+                    newvalue[0]
                 else
-                    @should.join(",")
+                    newvalue.join(",")
                 end
             else
                 nil
             end
         end
 
-        def is=(val)
-            if val.is_a?(Array)
-                @is = val
-            else
-                @is = [val]
-            end
-        end
-
-        def is_to_s
-            if @is
-                unless @is.is_a?(Array)
-                    return @is
+        def is_to_s(currentvalue = @is)
+            if currentvalue
+                unless currentvalue.is_a?(Array)
+                    return currentvalue
                 end
 
-                if self.name == :command or @is[0].is_a? Symbol
-                    @is[0]
+                if self.name == :command or currentvalue[0].is_a? Symbol
+                    currentvalue[0]
                 else
-                    @is.join(",")
+                    currentvalue.join(",")
                 end
             else
                 nil
@@ -164,6 +173,11 @@ Puppet::Type.newtype(:cron) do
                 return value
             end
 
+            # Allow ranges + */2
+            if value =~ /^[0-9]+-[0-9]+\/[0-9]+$/
+                return value
+            end
+
             if value == "*"
                 return value
             end
@@ -190,7 +204,7 @@ Puppet::Type.newtype(:cron) do
     end
 
     # Somewhat uniquely, this property does not actually change anything -- it
-    # just calls +@parent.sync+, which writes out the whole cron tab for
+    # just calls +@resource.sync+, which writes out the whole cron tab for
     # the user in question.  There is no real way to change individual cron
     # jobs without rewriting the entire cron file.
     #
@@ -202,16 +216,17 @@ Puppet::Type.newtype(:cron) do
             best to always provide a fully qualified command.  The user's
             profile is not sourced when the command is run, so if the
             user's environment is desired it should be sourced manually.
-            
+
             All cron parameters support ``absent`` as a value; this will
             remove any existing values for that field."
 
-        def is
-            if @is
-                @is[0]
-            else
-                nil
-            end
+        def retrieve
+          return_value = super
+          if return_value && return_value.is_a?(Array)
+            return_value = return_value[0]
+          end
+
+          return return_value
         end
 
         def should
@@ -258,10 +273,10 @@ Puppet::Type.newtype(:cron) do
         def alpha
             %w{sunday monday tuesday wednesday thursday friday saturday}
         end
-        self.boundaries = [0, 6]
+        self.boundaries = [0, 7]
         desc "The weekday on which to run the command.
-            Optional; if specified, must be between 0 and 6, inclusive, with
-            0 being Sunday, or must be the name of the day (e.g., Tuesday)."
+            Optional; if specified, must be between 0 and 7, inclusive, with
+            0 (or 7) being Sunday, or must be the name of the day (e.g., Tuesday)."
     end
 
     newproperty(:month, :parent => CronParam) do
@@ -286,33 +301,54 @@ Puppet::Type.newtype(:cron) do
             can be no guarantees that other, earlier settings will not also
             affect a given cron job.
 
+
             Also, Puppet cannot automatically determine whether an existing,
             unmanaged environment setting is associated with a given cron
             job.  If you already have cron jobs with environment settings,
             then Puppet will keep those settings in the same place in the file,
             but will not associate them with a specific job.
-            
+
             Settings should be specified exactly as they should appear in
             the crontab, e.g., ``PATH=/bin:/usr/bin:/usr/sbin``."
 
         validate do |value|
-            unless value =~ /^\s*(\w+)\s*=\s*(.+)\s*$/
+            unless value =~ /^\s*(\w+)\s*=\s*(.*)\s*$/ or value == :absent or value == "absent"
                 raise ArgumentError, "Invalid environment setting %s" %
                     value.inspect
             end
         end
 
-        def insync?
-            if @is.is_a? Array
-                return @is.sort == @should.sort
+        def insync?(is)
+            if is.is_a? Array
+                return is.sort == @should.sort
             else
-                return @is == @should
+                return is == @should
+            end
+        end
+
+        def is_to_s(newvalue)
+            if newvalue
+                if newvalue.is_a?(Array)
+                    newvalue.join(",")
+                else
+                    newvalue
+                end
+            else
+                nil
             end
         end
 
         def should
             @should
         end
+
+        def should_to_s(newvalue = @should)
+            if newvalue
+                newvalue.join(",")
+            else
+                nil
+            end
+        end
     end
 
     newparam(:name) do
@@ -332,10 +368,10 @@ Puppet::Type.newtype(:cron) do
         desc "The user to run the command as.  This user must
             be allowed to run cron jobs, which is not currently checked by
             Puppet.
-            
+
             The user defaults to whomever Puppet is running as."
 
-        defaultto { ENV["USER"] || "root" }
+        defaultto { Etc.getpwuid(Process.uid).name || "root" }
     end
 
     newproperty(:target) do
@@ -344,8 +380,8 @@ Puppet::Type.newtype(:cron) do
             Other providers default accordingly."
 
         defaultto {
-            if provider.is_a?(@parent.class.provider(:crontab))
-                if val = @parent.should(:user)
+            if provider.is_a?(@resource.class.provider(:crontab))
+                if val = @resource.should(:user)
                     val
                 else
                     raise ArgumentError,
@@ -370,7 +406,7 @@ Puppet::Type.newtype(:cron) do
             ret = obj.should
 
             if ret.nil?
-                ret = obj.is
+                ret = obj.retrieve
             end
 
             if ret == :absent
@@ -394,4 +430,5 @@ Puppet::Type.newtype(:cron) do
     end
 end
 
-# $Id: cron.rb 2408 2007-04-24 02:41:16Z luke $
+
+
