@@ -1,12 +1,12 @@
 module Puppet
-    Puppet.type(:file).newproperty(:type) do
+    Puppet::Type.type(:file).newproperty(:type) do
         require 'etc'
         desc "A read-only state to check the file type."
 
         #munge do |value|
         #    raise Puppet::Error, ":type is read-only"
         #end
-        
+
         def retrieve
             currentvalue = :absent
             if stat = @resource.stat(false)
