@@ -2,15 +2,15 @@ module Puppet
     # We want the mount to refresh when it changes.
     newtype(:mount, :self_refresh => true) do
         @doc = "Manages mounted filesystems, including putting mount
-            information into the mount table. The actual behavior depends 
+            information into the mount table. The actual behavior depends
             on the value of the 'ensure' parameter.
-	    
+
             Note that if a ``mount`` receives an event from another resource,
             it will try to remount the filesystems if ``ensure`` is set to ``mounted``."
 
         feature :refreshable, "The provider can remount the filesystem.",
             :methods => [:remount]
-        
+
         # Use the normal parent class, because we actually want to
         # call code when sync() is called.
         newproperty(:ensure) do
@@ -44,7 +44,7 @@ module Puppet
             newvalue(:mounted, :event => :mount_mounted) do
                 # Create the mount point if it does not already exist.
                 current_value = self.retrieve
-                provider.create if current_value.nil? or current_value == :absent 
+                provider.create if current_value.nil? or current_value == :absent
 
                 syncothers()
 
@@ -54,7 +54,7 @@ module Puppet
 
             def retrieve
                 # We need to special case :mounted; if we're absent, we still
-                # want 
+                # want
                 curval = super()
                 if curval == :absent
                     return curval
@@ -95,7 +95,7 @@ module Puppet
 
         # Solaris specifies two devices, not just one.
         newproperty(:blockdevice) do
-            desc "The the device to fsck.  This is property is only valid
+            desc "The device to fsck.  This is property is only valid
                 on Solaris, and in most cases will default to the correct
                 value."
 
@@ -140,8 +140,16 @@ module Puppet
         end
 
         newproperty(:dump) do
-            desc "Whether to dump the mount.  Not all platforms
-                support this."
+            desc "Whether to dump the mount.  Not all platform support this. 
+                Valid values are ``1`` or ``0``. or ``2`` on FreeBSD, Default is ``0``." 
+            
+            if Facter["operatingsystem"].value == "FreeBSD" 
+                newvalue(%r{(0|1|2)})
+            else
+                newvalue(%r{(0|1)})
+            end
+
+            newvalue(%r{(0|1)})
 
             defaultto {
                 if @resource.managed?
@@ -152,7 +160,7 @@ module Puppet
 
         newproperty(:target) do
             desc "The file in which to store the mount table.  Only used by
-                those providers that write to disk (i.e., not NetInfo)."
+                those providers that write to disk."
 
             defaultto { if @resource.class.defaultprovider.ancestors.include?(Puppet::Provider::ParsedFile)
                     @resource.class.defaultprovider.default_target
@@ -177,16 +185,17 @@ module Puppet
                 super
             end
         end
-        
+
         newparam(:remounts) do
             desc "Whether the mount can be remounted  ``mount -o remount``.  If
                 this is false, then the filesystem will be unmounted and remounted
                 manually, which is prone to failure."
-            
+
             newvalues(:true, :false)
             defaultto do
                 case Facter.value(:operatingsystem)
-                when "FreeBSD": false
+                when "FreeBSD", "Darwin"
+                    false
                 else
                     true
                 end
