@@ -9,11 +9,11 @@ module Puppet
         newproperty(:message) do
             desc "The message to be sent to the log."
             def sync
-                case @parent["withpath"]
-                when :true:
-                    log(self.should)
-                else  
-                    Puppet.send(@parent[:loglevel], self.should)
+                case @resource["withpath"]
+                when :true
+                    send(@resource[:loglevel], self.should)
+                else
+                    Puppet.send(@resource[:loglevel], self.should)
                 end
                 return
             end
@@ -22,19 +22,18 @@ module Puppet
                 return
             end
 
-            def insync?
+            def insync?(is)
                 false
             end
 
-            defaultto { @parent[:name] }
+            defaultto { @resource[:name] }
         end
 
-        newparam(:withpath) do 
-            desc "Whether to not to show the full object path.  Sends the
-                message at the current loglevel."
+        newparam(:withpath) do
+            desc "Whether to not to show the full object path."
             defaultto :false
 
-            newvalues(:true, :false) 
+            newvalues(:true, :false)
         end
 
         newparam(:name) do
@@ -44,4 +43,3 @@ module Puppet
     end
 end
 
-# $Id: notify.rb 2317 2007-03-19 17:42:17Z luke $
