@@ -1,12 +1,15 @@
 require 'etc'
 require 'facter'
+require 'puppet/property/list'
+require 'puppet/property/ordered_list'
+require 'puppet/property/keyvalue'
 
 module Puppet
     newtype(:user) do
         @doc = "Manage users.  This type is mostly built to manage system
             users, so it is lacking some features useful for managing normal
             users.
-            
+
             This resource type uses the prescribed native tools for creating
             groups and generally uses POSIX APIs for retrieving information
             about them.  It does not directly modify /etc/passwd or anything."
@@ -21,6 +24,9 @@ module Puppet
             "The provider can modify user passwords, by accepting a password
             hash."
 
+        feature :manages_solaris_rbac,
+            "The provider can manage roles and normal users"
+
         newproperty(:ensure, :parent => Puppet::Property::Ensure) do
             newvalue(:present, :event => :user_created) do
                 provider.create
@@ -30,6 +36,10 @@ module Puppet
                 provider.delete
             end
 
+            newvalue(:role, :event => :role_created, :required_features => :manages_solaris_rbac) do
+                provider.create_role
+            end
+
             desc "The basic state that the object should be in."
 
             # If they're talking about the thing at all, they generally want to
@@ -44,30 +54,15 @@ module Puppet
 
             def retrieve
                 if provider.exists?
-                    return :present
+                    if provider.respond_to?(:is_role?) and provider.is_role?
+                        return :role
+                    else
+                        return :present
+                    end
                 else
                     return :absent
                 end
             end
-
-            # The default 'sync' method only selects among a list of registered
-            # values.
-            def sync
-#                if self.insync?
-#                    self.info "already in sync"
-#                    return nil
-                #else
-                    #self.info "%s vs %s" % [self.is.inspect, self.should.inspect]
-#               end
-                unless self.class.values
-                    self.devfail "No values defined for %s" %
-                        self.class.name
-                end
-
-                # Set ourselves to whatever our should value is.
-                self.set(self.should)
-            end
-
         end
 
         newproperty(:uid) do
@@ -95,50 +90,40 @@ module Puppet
         newproperty(:gid) do
             desc "The user's primary group.  Can be specified numerically or
                 by name."
-            
-            def found?
-                defined? @found and @found
-            end
 
-            munge do |gid|
-                method = :getgrgid
-                case gid
-                when String
-                    if gid =~ /^[-0-9]+$/
-                        gid = Integer(gid)
-                    else
-                        method = :getgrnam
-                    end
-                when Symbol
-                    unless gid == :auto or gid == :absent
-                        self.devfail "Invalid GID %s" % gid
-                    end
-                    # these are treated specially by sync()
-                    return gid
-                end
-
-                if group = Puppet::Util.gid(gid)
-                    @found = true
-                    return group
+            munge do |value|
+                if value.is_a?(String) and value =~ /^[-0-9]+$/
+                    Integer(value)
                 else
-                    @found = false
-                    return gid
+                    value
                 end
             end
 
-            # *shudder*  Make sure that we've looked up the group and gotten
-            # an ID for it.  Yuck-o.
-            def should
-                unless defined? @should
-                    return super
+            def insync?(is)
+                return true unless self.should
+
+                # We know the 'is' is a number, so we need to convert the 'should' to a number,
+                # too.
+                @should.each do |value|
+                    return true if number = Puppet::Util.gid(value) and is == number
                 end
-                unless found?
-                    @should = @should.each { |val|
-                        next unless val
-                        Puppet::Util.gid(val)
-                    }
+
+                return false
+            end
+
+            def sync
+                found = false
+                @should.each do |value|
+                    if number = Puppet::Util.gid(value)
+                        provider.gid = number
+                        found = true
+                        break
+                    end
                 end
-                super
+
+                fail "Could not find group(s) %s" % @should.join(",") unless found
+
+                # Use the default event.
             end
         end
 
@@ -158,94 +143,36 @@ module Puppet
 
         newproperty(:password, :required_features => :manages_passwords) do
             desc "The user's password, in whatever encrypted format the local machine requires. Be sure to enclose any value that includes a dollar sign ($) in single quotes (\')."
+
+            validate do |value|
+                raise ArgumentError, "Passwords cannot include ':'" if value.is_a?(String) and value.include?(":")
+            end
+
+            def change_to_s(currentvalue, newvalue)
+                if currentvalue == :absent
+                  return "created password"
+                else
+                  return "changed password"
+                end
+            end
         end
 
-        newproperty(:groups) do
+
+        newproperty(:groups, :parent => Puppet::Property::List) do
             desc "The groups of which the user is a member.  The primary
                 group should not be listed.  Multiple groups should be
                 specified as an array."
 
-            def should_to_s(newvalue)
-                self.should
-            end
-
-            def is_to_s(currentvalue)
-                currentvalue.join(",")
-            end
-
-            # We need to override this because the groups need to
-            # be joined with commas
-            def should
-                current_value = retrieve
-
-                unless defined? @should and @should
-                    return nil
-                end
-
-                if @resource[:membership] == :inclusive
-                    return @should.sort.join(",")
-                else
-                    members = @should
-                    if current_value.is_a?(Array)
-                        members += current_value
-                    end
-                    return members.uniq.sort.join(",")
-                end
-            end
-
-            def retrieve
-                if tmp = provider.groups and tmp != :absent
-                    return tmp.split(",")
-                else
-                    return :absent
-                end
-            end
-
-            def insync?(is)
-                unless defined? @should and @should
-                    return true
-                end
-                unless defined? is and is
-                    return true
-                end
-                tmp = is
-                if is.is_a? Array
-                    tmp = is.sort.join(",")
-                end
-
-                return tmp == self.should
-            end
-
             validate do |value|
                 if value =~ /^\d+$/
                     raise ArgumentError, "Group names must be provided, not numbers"
                 end
+                if value.include?(",")
+                    raise ArgumentError, "Group names must be provided as an array, not a comma-separated list"
+                end
             end
         end
 
-        # these three properties are all implemented differently on each platform,
-        # so i'm disabling them for now
-
-        # FIXME Puppet::Property::UserLocked is currently non-functional
-        #newproperty(:locked) do
-        #    desc "The expected return code.  An error will be returned if the
-        #        executed command returns something else."
-        #end
-
-        # FIXME Puppet::Property::UserExpire is currently non-functional
-        #newproperty(:expire) do
-        #    desc "The expected return code.  An error will be returned if the
-        #        executed command returns something else."
-        #    @objectaddflag = "-e"
-        #end
-
-        # FIXME Puppet::Property::UserInactive is currently non-functional
-        #newproperty(:inactive) do
-        #    desc "The expected return code.  An error will be returned if the
-        #        executed command returns something else."
-        #    @objectaddflag = "-f"
-        #end
-
         newparam(:name) do
             desc "User name.  While limitations are determined for
                 each operating system, it is generally a good idea to keep to
@@ -257,7 +184,7 @@ module Puppet
             desc "Whether specified groups should be treated as the only groups
                 of which the user is a member or whether they should merely
                 be treated as the minimum membership list."
-                
+
             newvalues(:inclusive, :minimum)
 
             defaultto :minimum
@@ -265,7 +192,7 @@ module Puppet
 
         newparam(:allowdupe, :boolean => true) do
             desc "Whether to allow duplicate UIDs."
-                
+
             newvalues(:true, :false)
 
             defaultto false
@@ -301,12 +228,9 @@ module Puppet
                 }
                 groups.each { |group|
                     case group
-                    when Integer:
-                        if obj = Puppet.type(:group).find { |gobj|
-                            gobj.should(:gid) == group
-                        }
-                            autos << obj
-                            
+                    when Integer
+                        if resource = catalog.resources.find { |r| r.is_a?(Puppet::Type.type(:group)) and r.should(:gid) == group }
+                            autos << resource
                         end
                     else
                         autos << group
@@ -321,13 +245,18 @@ module Puppet
             autos
         end
 
+        # Provide an external hook.  Yay breaking out of APIs.
+        def exists?
+            provider.exists?
+        end
+
         def retrieve
             absent = false
             properties().inject({}) { |prophash, property|
                 current_value = :absent
 
                 if absent
-                   prophash[property] = :absent
+                    prophash[property] = :absent
                 else
                     current_value = property.retrieve
                     prophash[property] = current_value
@@ -335,11 +264,133 @@ module Puppet
 
                 if property.name == :ensure and current_value == :absent
                     absent = true
-#                    next
                 end
                 prophash
             }
         end
+
+        newproperty(:roles, :parent => Puppet::Property::List, :required_features => :manages_solaris_rbac) do
+            desc "The roles the user has.  Multiple roles should be
+                specified as an array."
+
+            def membership
+                :role_membership
+            end
+
+            validate do |value|
+                if value =~ /^\d+$/
+                    raise ArgumentError, "Role names must be provided, not numbers"
+                end
+                if value.include?(",")
+                    raise ArgumentError, "Role names must be provided as an array, not a comma-separated list"
+                end
+            end
+        end
+
+        #autorequire the roles that the user has
+        autorequire(:user) do
+            reqs = []
+
+            if roles_property = @parameters[:roles] and roles = roles_property.should
+                reqs += roles.split(',')
+            end
+
+            reqs
+        end
+
+        newparam(:role_membership) do
+            desc "Whether specified roles should be treated as the only roles
+                of which the user is a member or whether they should merely
+                be treated as the minimum membership list."
+
+            newvalues(:inclusive, :minimum)
+
+            defaultto :minimum
+        end
+
+        newproperty(:auths, :parent => Puppet::Property::List, :required_features => :manages_solaris_rbac) do
+            desc "The auths the user has.  Multiple auths should be
+                specified as an array."
+
+            def membership
+                :auth_membership
+            end
+
+            validate do |value|
+                if value =~ /^\d+$/
+                    raise ArgumentError, "Auth names must be provided, not numbers"
+                end
+                if value.include?(",")
+                    raise ArgumentError, "Auth names must be provided as an array, not a comma-separated list"
+                end
+            end
+        end
+
+        newparam(:auth_membership) do
+            desc "Whether specified auths should be treated as the only auths
+                of which the user is a member or whether they should merely
+                be treated as the minimum membership list."
+
+            newvalues(:inclusive, :minimum)
+
+            defaultto :minimum
+        end
+
+        newproperty(:profiles, :parent => Puppet::Property::OrderedList, :required_features => :manages_solaris_rbac) do
+            desc "The profiles the user has.  Multiple profiles should be
+                specified as an array."
+
+            def membership
+                :profile_membership
+            end
+
+            validate do |value|
+                if value =~ /^\d+$/
+                    raise ArgumentError, "Profile names must be provided, not numbers"
+                end
+                if value.include?(",")
+                    raise ArgumentError, "Profile names must be provided as an array, not a comma-separated list"
+                end
+            end
+        end
+
+        newparam(:profile_membership) do
+            desc "Whether specified roles should be treated as the only roles
+                of which the user is a member or whether they should merely
+                be treated as the minimum membership list."
+
+            newvalues(:inclusive, :minimum)
+
+            defaultto :minimum
+        end
+
+        newproperty(:keys, :parent => Puppet::Property::KeyValue, :required_features => :manages_solaris_rbac) do
+            desc "Specify user attributes in an array of keyvalue pairs"
+
+            def membership
+                :key_membership
+            end
+
+            validate do |value|
+                unless value.include?("=")
+                    raise ArgumentError, "key value pairs must be seperated by an ="
+                end
+            end
+        end
+
+        newparam(:key_membership) do
+            desc "Whether specified key value pairs should be treated as the only attributes
+                of the user or whether they should merely
+                be treated as the minimum list."
+
+            newvalues(:inclusive, :minimum)
+
+            defaultto :minimum
+        end
+
+        newproperty(:project, :required_features => :manages_solaris_rbac) do
+            desc "The name of the project associated with a user"
+        end
     end
 end
 
