@@ -1,8 +1,13 @@
 # Basic classes for reading, writing, and emptying files.  Not much
 # to see here.
+
+require 'puppet/util/selinux'
+
 class Puppet::Util::FileType
     attr_accessor :loaded, :path, :synced
 
+    include Puppet::Util::SELinux
+
     class << self
         attr_accessor :name
         include Puppet::Util::ClassGen
@@ -67,15 +72,12 @@ class Puppet::Util::FileType
         @filetypes[type]
     end
 
-    # Back the file up before replacing it.
-    def backup
-        bucket.backup(@path) if FileTest.exists?(@path)
-    end
-
     # Pick or create a filebucket to use.
     def bucket
-        filebucket = Puppet::Type.type(:filebucket)
-        (filebucket["puppet"] || filebucket.mkdefaultbucket).bucket
+        unless defined?(@bucket)
+            @bucket = Puppet::Type.type(:filebucket).mkdefaultbucket.bucket
+        end
+        @bucket
     end
 
     def initialize(path)
@@ -85,9 +87,14 @@ class Puppet::Util::FileType
 
     # Operate on plain files.
     newfiletype(:flat) do
+        # Back the file up before replacing it.
+        def backup
+            bucket.backup(@path) if File.exists?(@path)
+        end
+
         # Read the file.
         def read
-            if File.exists?(@path)
+            if File.exist?(@path)
                 File.read(@path)
             else
                 return nil
@@ -96,19 +103,20 @@ class Puppet::Util::FileType
 
         # Remove the file.
         def remove
-            if File.exists?(@path)
+            if File.exist?(@path)
                 File.unlink(@path)
             end
         end
 
         # Overwrite the file.
         def write(text)
-            backup()
             require "tempfile"
-            tf = Tempfile.new("puppet") 
-            tf.print text; tf.flush 
-            FileUtils.cp(tf.path, @path) 
+            tf = Tempfile.new("puppet")
+            tf.print text; tf.flush
+            FileUtils.cp(tf.path, @path)
             tf.close
+            # If SELinux is present, we need to ensure the file has its expected context
+            set_selinux_default_context(@path)
         end
     end
 
@@ -206,10 +214,10 @@ class Puppet::Util::FileType
             begin
                 output = Puppet::Util.execute(%w{crontab -l}, :uid => @path)
                 return "" if output.include?("can't open your crontab")
+                raise Puppet::Error, "User %s not authorized to use cron" % @path if output.include?("you are not authorized to use cron")
                 return output
-            rescue
-                # If there's a failure, treat it like an empty file.
-                return ""
+            rescue => detail
+                raise Puppet::Error, "Could not read crontab for %s: %s" % [@path, detail]
             end
         end
 
@@ -244,91 +252,49 @@ class Puppet::Util::FileType
         end
     end
 
-    # Treat netinfo tables as a single file, just for simplicity of certain
-    # types
-    newfiletype(:netinfo) do
-        class << self
-            attr_accessor :format
-        end
+    #  Support for AIX crontab with output different than suntab's crontab command.
+    newfiletype(:aixtab) do
+        # Read a specific @path's cron tab.
         def read
-            %x{nidump -r /#{@path} /}
+            begin
+                output = Puppet::Util.execute(%w{crontab -l}, :uid => @path)
+                if output.include?("You are not authorized to use the cron command")
+                    raise Puppet::Error, "User %s not authorized to use cron" % @path 
+                end
+                return output
+            rescue => detail
+                raise Puppet::Error, "Could not read crontab for %s: %s" % [@path, detail]
+            end
         end
 
-        # This really only makes sense for cron tabs.
+        # Remove a specific @path's cron tab.
         def remove
-            %x{nireport / /#{@path} name}.split("\n").each do |name|
-                newname = name.gsub(/\//, '\/').sub(/\s+$/, '')
-                output = %x{niutil -destroy / '/#{@path}/#{newname}'}
-
-                unless $? == 0
-                    raise Puppet::Error, "Could not remove %s from %s" %
-                        [name, @path]
-                end
+            begin
+                Puppet::Util.execute(%w{crontab -r}, :uid => @path)
+            rescue => detail
+                raise Puppet::Error, "Could not remove crontab for %s: %s" % [@path, detail]
             end
         end
 
-        # Convert our table to an array of hashes.  This only works for
-        # handling one table at a time.
-        def to_array(text = nil)
-            unless text
-                text = read
-            end
-
-            hash = nil
-
-            # Initialize it with the first record
-            records = []
-            text.split("\n").each do |line|
-                next if line =~ /^[{}]$/ # Skip the wrapping lines
-                next if line =~ /"name" = \( "#{@path}" \)/ # Skip the table name
-                next if line =~ /CHILDREN = \(/ # Skip this header
-                next if line =~ /^  \)/ # and its closer
-
-                # Now we should have nothing but records, wrapped in braces
-
-                case line
-                when /^\s+\{/: hash = {}
-                when /^\s+\}/: records << hash
-                when /\s+"(\w+)" = \( (.+) \)/
-                    field = $1
-                    values = $2
-
-                    # Always use an array
-                    hash[field] = []
-
-                    values.split(/, /).each do |value|
-                        if value =~ /^"(.*)"$/
-                            hash[field] << $1
-                        else
-                            raise ArgumentError, "Could not match value %s" % value
-                        end
-                    end
-                else    
-                    raise ArgumentError, "Could not match line %s" % line
-                end
-            end
-
-            records
-        end
-
+        # Overwrite a specific @path's cron tab; must be passed the @path name
+        # and the text with which to create the cron tab.
         def write(text)
-            text.gsub!(/^#.*\n/,'')
-            text.gsub!(/^$/,'')
-            if text == "" or text == "\n"
-                self.remove
-                return
-            end
-            unless format = self.class.format
-                raise Puppe::DevError, "You must define the NetInfo format to inport"
-            end
-            IO.popen("niload -d #{format} . 1>/dev/null 2>/dev/null", "w") { |p|
-                p.print text
-            }
+            require "tempfile"
+            output_file = Tempfile.new("puppet")
+            fh = output_file.open
+            fh.print text
+            fh.close
 
-            unless $? == 0
-                raise ArgumentError, "Failed to write %s" % @path
+            # We have to chown the stupid file to the user.
+            File.chown(Puppet::Util.uid(@path), nil, output_file.path)
+
+            begin
+                Puppet::Util.execute(["crontab", output_file.path], :uid => @path)
+            rescue => detail
+                raise Puppet::Error, "Could not write crontab for %s: %s" % [@path, detail]
+            ensure
+                output_file.delete
             end
         end
     end
 end
-
