@@ -1,10 +1,12 @@
 require 'syslog'
+require 'puppet/util/tagging'
 
 # Pass feedback to the user.  Log levels are modeled after syslog's, and it is
 # expected that that will be the most common log destination.  Supports
 # multiple destinations, one of which is a remote server.
 class Puppet::Util::Log
     include Puppet::Util
+    include Puppet::Util::Tagging
 
     @levels = [:debug,:info,:notice,:warning,:err,:alert,:emerg,:crit]
     @loglevel = 2
@@ -102,6 +104,13 @@ class Puppet::Util::Log
         end
     end
 
+    def self.close_all
+        # And close all logs except the console.
+        destinations.each do |dest|
+            close(dest)
+        end
+    end
+
     # Flush any log destinations that support such operations.
     def Log.flush
         @destinations.each { |type, dest|
@@ -120,7 +129,7 @@ class Puppet::Util::Log
         unless @levels.index(hash[:level])
             raise Puppet::DevError, "Invalid log level %s" % hash[:level]
         end
-        if @levels.index(hash[:level]) >= @loglevel 
+        if @levels.index(hash[:level]) >= @loglevel
             return Puppet::Util::Log.new(hash)
         else
             return nil
@@ -242,39 +251,47 @@ class Puppet::Util::Log
     end
 
     newdesttype :console do
-                
-        
-        PINK = {:console => "[0;31m", :html => "FFA0A0"}
-        GREEN = {:console => "[0;32m", :html => "00CD00"}
-        YELLOW = {:console => "[0;33m", :html => "FFFF60"}
-        SLATE = {:console => "[0;34m", :html => "80A0FF"}
-        ORANGE = {:console => "[0;35m", :html => "FFA500"}
-        BLUE = {:console => "[0;36m", :html => "40FFFF"}
-        RESET = {:console => "[0m", :html => ""}
+
+
+        RED     = {:console => "[0;31m", :html => "FFA0A0"}
+        GREEN   = {:console => "[0;32m", :html => "00CD00"}
+        YELLOW  = {:console => "[0;33m", :html => "FFFF60"}
+        BLUE    = {:console => "[0;34m", :html => "80A0FF"}
+        PURPLE  = {:console => "[0;35m", :html => "FFA500"}
+        CYAN    = {:console => "[0;36m", :html => "40FFFF"}
+        WHITE   = {:console => "[0;37m", :html => "FFFFFF"}
+        HRED    = {:console => "[1;31m", :html => "FFA0A0"}
+        HGREEN  = {:console => "[1;32m", :html => "00CD00"}
+        HYELLOW = {:console => "[1;33m", :html => "FFFF60"}
+        HBLUE   = {:console => "[1;34m", :html => "80A0FF"}
+        HPURPLE = {:console => "[1;35m", :html => "FFA500"}
+        HCYAN   = {:console => "[1;36m", :html => "40FFFF"}
+        HWHITE  = {:console => "[1;37m", :html => "FFFFFF"}
+        RESET   = {:console => "[0m",    :html => ""      }
 
         @@colormap = {
-            :debug => SLATE,
+            :debug => WHITE,
             :info => GREEN,
-            :notice => PINK,
-            :warning => ORANGE,
-            :err => YELLOW,
-            :alert => BLUE,
-            :emerg => RESET,
-            :crit => RESET
+            :notice => CYAN,
+            :warning => YELLOW,
+            :err => HPURPLE,
+            :alert => RED,
+            :emerg => HRED,
+            :crit => HRED
         }
-        
+
         def colorize(level, str)
             case Puppet[:color]
-            when false: str
-            when true, :ansi, "ansi": console_color(level, str)
-            when :html, "html": html_color(level, str)
+            when true, :ansi, "ansi", :yes, "yes"; console_color(level, str)
+            when :html, "html"; html_color(level, str)
+            else str
             end
         end
-        
+
         def console_color(level, str)
             @@colormap[level][:console] + str + RESET[:console]
         end
-        
+
         def html_color(level, str)
             %{<span style="color: %s">%s</span>} % [@@colormap[level][:html], str]
         end
@@ -359,11 +376,7 @@ class Puppet::Util::Log
         end
 
         def handle(msg)
-            # Only add messages from objects, since anything else is
-            # probably unrelated to this run.
-            if msg.objectsource?
-                @report.newlog(msg)
-            end
+            @report.newlog(msg)
         end
     end
 
@@ -419,7 +432,7 @@ class Puppet::Util::Log
     # for a loop here, if the machine somehow gets the destination set as
     # itself.
     def Log.newmessage(msg)
-        if @levels.index(msg.level) < @loglevel 
+        if @levels.index(msg.level) < @loglevel
             return
         end
 
@@ -431,7 +444,7 @@ class Puppet::Util::Log
     end
 
     def Log.sendlevel?(level)
-        @levels.index(level) >= @loglevel 
+        @levels.index(level) >= @loglevel
     end
 
     # Reopen all of our logs.
@@ -462,12 +475,12 @@ class Puppet::Util::Log
         @levels.include?(level)
     end
 
-    attr_accessor :level, :message, :time, :tags, :remote
+    attr_accessor :level, :message, :time, :remote, :file, :line, :version
     attr_reader :source
 
     def initialize(args)
         unless args.include?(:level) && args.include?(:message)
-            raise Puppet::DevError, "Puppet::Util::Log called incorrectly"
+            raise ArgumentError, "Puppet::Util::Log called incorrectly"
         end
 
         if args[:level].class == String
@@ -475,65 +488,42 @@ class Puppet::Util::Log
         elsif args[:level].class == Symbol
             @level = args[:level]
         else
-            raise Puppet::DevError,
-                "Level is not a string or symbol: #{args[:level].class}"
+            raise ArgumentError, "Level is not a string or symbol: #{args[:level].class}"
         end
 
-        # Just return unless we're actually at a level we should send
-        #return unless self.class.sendlevel?(@level)
-
         @message = args[:message].to_s
         @time = Time.now
-        # this should include the host name, and probly lots of other
-        # stuff, at some point
-        unless self.class.validlevel?(level)
-            raise Puppet::DevError, "Invalid message level #{level}"
+
+        raise ArgumentError, "Invalid log level %s" % level unless self.class.validlevel?(level)
+
+        if tags = args[:tags]
+            tags.each { |t| self.tag(t) }
         end
 
-        if args.include?(:tags)
-            @tags = args[:tags]
-        end
+        self.source = args[:source] || "Puppet"
 
-        if args.include?(:source)
-            self.source = args[:source]
-        else
-            @source = "Puppet"
-        end
+        # Tag myself with my log level
+        tag(level)
 
         Log.newmessage(self)
     end
 
-    # Was the source of this log an object?
-    def objectsource?
-        if defined? @objectsource and @objectsource
-            @objectsource
-        else
-            false
-        end
-    end
-
     # If they pass a source in to us, we make sure it is a string, and
     # we retrieve any tags we can.
     def source=(source)
-        # We can't store the actual source, we just store the path.
-        # We can't just check for whether it responds to :path, because
-        # plenty of providers respond to that in their normal function.
-        if (source.is_a?(Puppet::Type) or source.is_a?(Puppet::Parameter)) and source.respond_to?(:path)
-            @objectsource = true
-            @source = source.path
+        if source.respond_to?(:source_descriptors)
+            descriptors = source.source_descriptors
+            @source = descriptors[:path]
+
+            descriptors[:tags].each { |t| tag(t) }
+
+            [:file, :line, :version].each do |param|
+                next unless descriptors[param]
+                send(param.to_s + "=", descriptors[param])
+            end
         else
-            @objectsource = false
             @source = source.to_s
         end
-        unless defined? @tags and @tags
-            if source.respond_to?(:tags)
-                @tags = source.tags
-            end
-        end
-    end
-
-    def tagged?(tag)
-        @tags.detect { |t| t.to_s == tag.to_s }
     end
 
     def to_report
@@ -544,5 +534,8 @@ class Puppet::Util::Log
         return @message
     end
 end
-Puppet::Log = Puppet::Util::Log
 
+# This is for backward compatibility from when we changed the constant to Puppet::Util::Log
+# because the reports include the constant name.  Apparently the alias was created in
+# March 2007, should could probably be removed soon.
+Puppet::Log = Puppet::Util::Log
