@@ -1,11 +1,17 @@
 require 'yaml'
 require 'sync'
 
+require 'puppet/util/file_locking'
+
 # a class for storing state
 class Puppet::Util::Storage
     include Singleton
     include Puppet::Util
 
+    def self.state
+        return @@state
+    end
+
     def initialize
         self.class.load
     end
@@ -55,7 +61,7 @@ class Puppet::Util::Storage
             return
         end
         Puppet::Util.benchmark(:debug, "Loaded state") do
-            Puppet::Util.readlock(Puppet[:statefile]) do |file|
+            Puppet::Util::FileLocking.readlock(Puppet[:statefile]) do |file|
                 begin
                     @@state = YAML.load(file)
                 rescue => detail
@@ -93,7 +99,7 @@ class Puppet::Util::Storage
         end
 
         Puppet::Util.benchmark(:debug, "Stored state") do
-            Puppet::Util.writelock(Puppet[:statefile], 0660) do |file|
+            Puppet::Util::FileLocking.writelock(Puppet[:statefile], 0660) do |file|
                 file.print YAML.dump(@@state)
             end
         end
